@@ -3,20 +3,19 @@ package t::util;
 use Carp;
 use FindBin;
 use File::Basename qw(fileparse);
+use File::Spec;
 use Cwd qw(cwd);
 use Test::More;
-use Test::Deep;
-# Test::Deep doesn't export cmp_details, deep_diag until recent version (0.104)
-# so we import them "by hand"
-*cmp_details=\&Test::Deep::cmp_details;
-*deep_diag=\&Test::Deep::deep_diag;
+use Test::Deep qw(cmp_details deep_diag methods set);
 use Exporter();
 use strict;
 
 our @ISA=qw(Exporter);
-our @EXPORT=qw(script scriptpath scriptfullpath scriptbasename scriptcode rootpath
-	       as_bool flatten
-	       cmp_quietly cmp_attrs report report_pass report_fail
+our @EXPORT=qw(script scriptpath scriptfullpath scriptbasename scriptcode subtestdir rootpath
+	       as_bool as_list flatten
+	       is_quietly is_loudly cmp_quietly cmp_set_quietly cmp_attrs 
+	       report report_pass report_fail 
+	       called_from group
 	     );
 our($SCRIPT,$SCRIPTPATH,$SCRIPTBASENAME,$SCRIPTCODE,$ROOTPATH);
 sub script {$SCRIPT or (($SCRIPT,$SCRIPTPATH)=fileparse($0) and $SCRIPT);}
@@ -25,11 +24,26 @@ sub scriptfullpath {$FindBin::Bin}
 sub scriptbasename {$SCRIPTBASENAME or 
 		      (($SCRIPTBASENAME)=fileparse($0,qw(.t)) and $SCRIPTBASENAME);}
 sub scriptcode {$SCRIPTCODE or (($SCRIPTCODE)=script=~/\.(\w+)\.t$/)[0];}
+sub subtestdir {File::Spec->catdir(scriptpath,scriptbasename)}
 sub rootpath {$ROOTPATH or $ROOTPATH=cwd}
 
 sub as_bool {$_[0]? 1: 0}
+sub as_list {my $list=@_>1? [@_]: (ref $_[0]? $_[0]: ([split(/\s+/,$_[0])]))}
 sub flatten {map {'ARRAY' eq ref $_? @$_: $_} @_}
 
+# like is but reports errors the way we want
+sub is_loudly {
+  my($actual,$correct,$label,$file,$line)=@_;
+  my $ok=$actual eq $correct;
+  pass($label), return 1 if $ok;
+  report_fail($ok,"$label: expected $correct, got $actual",$file,$line);
+}
+# like is but reports errors the way we want
+sub is_quietly {
+  my($actual,$correct,$label,$file,$line)=@_;
+  report_fail($actual eq $correct,"$label: expected $correct, got $actual",$file,$line);
+}
+
 # like cmp_deeply but reports errors the way we want
 sub cmp_quietly {
   my($actual,$correct,$label,$file,$line)=@_;
@@ -38,6 +52,11 @@ sub cmp_quietly {
   my($ok,$details)=cmp_details($actual,$correct);
   report_fail($ok,"$label",$file,$line,$details);
 }
+# like cmp_set but reports errors the way we want
+sub cmp_set_quietly {
+  my($actual,$correct,$label,$file,$line)=@_;
+  cmp_quietly($actual,set(@$correct),$label,$file,$line);
+}
 # $actual is object. $correct is HASH of attr=>value pairs wih Test::Deep decorations
 sub cmp_attrs {
   my($actual,$correct,$label,$file,$line)=@_;
@@ -93,18 +112,18 @@ sub called_from {
 ################################################################################
 
 # # TODO: rewrite w/ Hash::AutoHash::MultiValued
-# # group a list by categories returned by sub.
-# # has to be declared before use, because of prototype
-# sub group (&@) {
-#   my($sub,@list)=@_;
-#   my %groups;
-#   for (@list) {
-#     my $group=&$sub($_);
-#     my $members=$groups{$group} || ($groups{$group}=[]);
-#     push(@$members,$_);
-#   }
-#   wantarray? %groups: \%groups;
-# }
+# group a list by categories returned by sub.
+# has to be declared before use, because of prototype
+sub group (&@) {
+  my($sub,@list)=@_;
+  my %groups;
+  for (@list) {
+    my $group=&$sub($_);
+    my $members=$groups{$group} || ($groups{$group}=[]);
+    push(@$members,$_);
+  }
+  wantarray? %groups: \%groups;
+}
 # # like group, but processes elements that are put on list. 
 # # sub should return 2 element list: 1st defines group, 2nd maps the value
 # # has to be declared before use, because of prototype
